package exnihilocreatio.client.renderers;

import exnihilocreatio.client.models.ModelVertex;
import exnihilocreatio.texturing.Color;
import exnihilocreatio.texturing.SpriteColor;
import exnihilocreatio.tiles.TileBarrel;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.texture.TextureAtlasSprite;
import net.minecraft.client.renderer.vertex.VertexFormatElement;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraftforge.client.model.animation.FastTESR;

import javax.annotation.Nullable;

public class RenderBarrel extends FastTESR<TileBarrel> {
    private static final ModelVertex[] model = new ModelVertex[4];

    static {
        model[0] = new ModelVertex(EnumFacing.UP, 0.125, 0.875, 0.125, 0, 0);
        model[1] = new ModelVertex(EnumFacing.UP, 0.875, 0.875, 0.125, 1, 0);
        model[2] = new ModelVertex(EnumFacing.UP, 0.875, 0.875, 0.875, 1, 1);
        model[3] = new ModelVertex(EnumFacing.UP, 0.125, 0.875, 0.875, 0, 1);
    }

    @Override
    public void renderTileEntityFast(@Nullable TileBarrel te, double x, double y, double z, float partialTicks, int destroyStage, float partial, @Nullable BufferBuilder buffer) {
        if (te == null || te.getMode() == null || buffer == null) return;

        // Fill Level
        float fill = te.getMode().getFilledLevelForRender(te);

        if (fill == 0) return;

        final SpriteColor sprite = te.getMode().getSpriteColor(te);
        buffer.func_178969_c(x, y, z);
        addSpriteColor(te, sprite, buffer, te.getMode().getFilledLevelForRender(te));
        buffer.func_178969_c(0, 0, 0);

    }

    private void addSpriteColor(TileBarrel te, SpriteColor sprite, BufferBuilder buffer, float fill) {
        if (sprite == null) return;

        final BlockPos pos = te.func_174877_v();
        // Light levels
        final int mixedBrightness = te.func_145831_w().func_180495_p(pos).func_185889_a(te.func_145831_w(), te.func_174877_v());
        final int skyLight = mixedBrightness >> 16 & 0xFFFF;
        final int blockLight = mixedBrightness & 0xFFFF;
        // Texturing
        TextureAtlasSprite icon = sprite.getSprite();
        Color color = sprite.getColor();

        // Draw
        for (final ModelVertex vert : model) {
            for (final VertexFormatElement e : buffer.func_178973_g().func_177343_g()) {
                switch (e.func_177375_c()) {
                    case COLOR:
                        buffer.func_181666_a(color.r, color.g, color.b, color.a);
                        break;

                    case NORMAL:
                        buffer.func_181663_c(vert.face.func_82601_c(), vert.face.func_96559_d(), vert.face.func_82599_e());
                        break;

                    case POSITION:
                        final double vertX = vert.x;
                        final double vertZ = vert.z;

                        buffer.func_181662_b(vertX, (double) fill, vertZ);
                        break;

                    case UV:
                        if (e.func_177369_e() == 1) {
                            buffer.func_187314_a(skyLight, blockLight);
                        } else {
                            buffer.func_187315_a(icon.func_94214_a(vert.u), icon.func_94207_b(16.0 - vert.v));
                        }
                        break;

                    default:
                        break;
                }
            }
            buffer.func_181675_d();
        }
    }
}
